@@ -517,6 +517,65 @@ class VarDelayFloat:
 		self.data[0] = value
 		return first_out
 
+@numba.jit
+def daqTriggerLogic(inputWaveform, rise, top, tau, triggerTreshold):
+	"""
+	This function emulated the Fast-DAQ system trigger logic. 
+	This is meant to be run over waveforms that were output from nabPy.bf.daqFilterEmulator and not over the normal waveforms. 
+	
+	Parameters
+	----------
+	inputWaveform: np.ndarray
+		The input waveform being examined. Expected to be in 8ns timebins from the daqFilterEmulation function
+	
+	rise: int
+		The rise time of the filter in 8ns timebins
+	
+	top: int
+		The flat top region of the filter in 8ns timebins
+	
+	tau: int
+		The decay rate of the filter in 8ns timebins
+	
+	triggerThreshold: int
+		The threshold by which the filter should arm itself in DAQ ADC units
+	
+	Returns
+	-------
+	results: list[tuple]
+		Returns a list of tuples. The first element in the tuple is the energy
+		The second element is the trigger timestamp.
+		
+		Waveforms may have multiple trigger locations hence the list instead of just a tuple
+	
+	"""
+	state = 0 #the current state of the system
+	curVal = 0
+	lastVal = 0
+	zeroCrossLoc = -1
+	wait = (rise//2) + (top*3)//4
+	output = []
+	for i in range(rise*3+top*2, len(inputWaveform)):
+		curVal = inputWaveform[i]
+		if state == 0: #searching for a trigger threshold
+			if abs(curVal) >= triggerTreshold:
+				state = 1
+		elif state == 1: #searching for a 0 cross
+			if np.sign(curVal) != np.sign(lastVal):
+				state = 2
+				zeroCrossLoc = i
+		elif state == 2:
+			if (i - zeroCrossLoc) > wait:
+				state = 3
+				output.append((curVal, (i-rise-(top*3)//4-top-rise)*2))
+		elif state == 3:
+			if abs(curVal) < triggerTreshold:
+				state = 0
+		else:
+			print('how did you get here???')
+		lastVal = curVal
+	return output
+
 @numba.jit
 def daqFilterEmulator(waveform, rise, top, tau, scaleFactor):
 	'''
@@ -586,7 +645,7 @@ def daqFilterEmulator(waveform, rise, top, tau, scaleFactor):
 		U = delay_u.Stuff(S)
 		acc3 += Q-acc2+S-U
 		Y = delay_y.Stuff(acc3)
-		output.append(delay_z.Stuff(np.rint(Y / scaleFactor)))
+		output.append(delay_z.Stuff(np.rint(Y * scaleFactor)))
 	return output
 
 def emulateDAQFilter(waves, rise, top, tau, triggerThreshold):
@@ -619,13 +678,14 @@ def emulateDAQFilter(waves, rise, top, tau, triggerThreshold):
 			if type(waves) == np.ndarray: #convert to dask array to do things in parallel
 				waves = da.asarray(waves)
 			#figure out the scale factor
-			scaleFactor = int(1.0/Fxp(1.0/(float(tau) * float(rise)), signed=False, n_word=24, n_frac=23))
-			filteredWaves = da.apply_along_axis(daqFilterEmulator, 1, waves, rise, top, tau, scaleFactor).compute()
+			scaleFactor = float(Fxp(1.0/(float(tau) * float(rise)), signed=False, n_word=24, n_frac=23))
+			filteredWaves = da.apply_along_axis(daqFilterEmulator, 1, waves, rise, top, tau, scaleFactor, dtype=np.int16)
+			
 			return filteredWaves
 		else:
 			print('fxpmath library not available. This is required for this function.')
 			print('Install from: https://github.com/francof2a/fxpmath ')
-			return [], []
+			return None
 
 @numba.jit
 def extractTrapResults(filtered, rise, top, percentage, shift = 0, mean = 0):
